@@ -45,6 +45,13 @@ sub new {
 	return $self;
 }
 
+# hack to have access to the store, to dump or restore it
+sub _store {
+	my $self = shift;
+	$self->{store} = shift if @_;
+	return $self->{store};
+}
+
 ###########################################################################
 # handle packet, called from Net::SIP::Dispatcher on incoming requests
 # Args: ($self,$packet,$leg,$addr)
@@ -62,23 +69,35 @@ sub receive {
 
 	# accept only REGISTER
 	$packet->is_request || return;
-	$packet->method eq 'REGISTER' || return;
+	if ( $packet->method ne 'REGISTER' ) {
+		# if we know the target rewrite the destination URI
+		my $addr = (sip_uri2parts($packet->uri))[3];
+		DEBUG( 1,"method ".$packet->method." addr=<$addr>" );
+		my @found = $self->query( $addr );
+		@found or do {
+			DEBUG( 1, "$addr not locally registered" );
+			return;
+		};
+		DEBUG( 1,"rewrite URI $addr in ".$packet->method." to $found[0]" );
+		$packet->set_uri( $found[0] );
+		return; # propagate to next in chain
+	}
 
-	my $from = $packet->get_header( 'from' ) or do {
-		DEBUG( 1,"no from in register request. DROP" );
+	my $to = $packet->get_header( 'to' ) or do {
+		DEBUG( 1,"no to in register request. DROP" );
 		return;
 	};
 
 	# what address will be registered
-	($from) = sip_hdrval2parts( from => $from );
-	if ( my ($domain,$user,$proto) = sip_uri2parts( $from ) ) {
+	($to) = sip_hdrval2parts( to => $to );
+	if ( my ($domain,$user,$proto) = sip_uri2parts( $to ) ) {
 		# normalize if possible
-		$from = "$proto:$user\@$domain";
+		$to = "$proto:$user\@$domain";
 	}
 
 	# check if domain is allowed
 	if ( my $rd = $self->{domains} ) {
-		my ($domain) = $from =~m{\@([\w\-\.]+)};
+		my ($domain) = $to =~m{\@([\w\-\.]+)};
 		if ( ! first { $domain =~m{\.?\Q$_\E$}i || $_ eq '*' } @$rd ) {
 			DEBUG( 1, "$domain matches none of my own domains. DROP" );
 			return;
@@ -92,9 +111,8 @@ sub receive {
 
 	# to which contacs it will be registered
 	my @contact = $packet->get_header( 'contact' );
-	my $store = $self->{store};
-	my $curr = $store->{ $from } ||= {};
 
+	my %curr;
 	foreach my $c (@contact) {
 		# update contact info
 		my ($c_addr,$param) = sip_hdrval2parts( contact => $c );
@@ -114,16 +132,18 @@ sub receive {
 			}
 			$expire += $now if $expire;
 		}
-		$curr->{$c_addr} = $expire;
+		$curr{$c_addr} = $expire;
 	}
 
+	$self->{store}{ $to } = \%curr;
+
 	# expire now!
 	$self->expire();
-	DEBUG_DUMP( 100,$store );
+	DEBUG_DUMP( 100,$self->{store} );
 
 	# send back a list of current contacts
 	my $response = $packet->create_response( '200','OK' );
-	while ( my ($where,$expire) = each %$curr ) {
+	while ( my ($where,$expire) = each %curr ) {
 		$expire -= $now;
 		$response->add_header( contact => "<$where>;expires=$expire" );
 	}
@@ -159,8 +179,8 @@ sub expire {
 	my $now = $loop->looptime;
 
 	my $store = $self->{store};
-	my (@drop_from,$next_exp);
-	while ( my ($from,$contact) = each %$store ) {
+	my (@drop_addr,$next_exp);
+	while ( my ($addr,$contact) = each %$store ) {
 		my @drop_where;
 		while ( my ($where,$expire) = each %$contact ) {
 			if ( $expire<$now ) {
@@ -171,10 +191,10 @@ sub expire {
 		}
 		if ( @drop_where ) {
 			delete @{$contact}{ @drop_where };
-			push @drop_from,$from if !%$contact;
+			push @drop_addr,$addr if !%$contact;
 		}
 	}
-	delete @{$store}{ @drop_from } if @drop_from;
+	delete @{$store}{ @drop_addr } if @drop_addr;
 
 	# add timer for next expire
 	if ( $next_exp ) {
